#include "grid_sync.h"
#include "math.h"
#include <hls_stream.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

#define ITERATIONS 2000
#define two_pi 6.283185307f

int main()
{
	/* Test signals */
	hls::stream<float> tb_in_Iga;
	hls::stream<float> tb_in_Igb;
	hls::stream<float> tb_in_Igc;
	hls::stream<float> tb_in_Uga;
	hls::stream<float> tb_in_Ugb;
	hls::stream<float> tb_in_Ugc;
	hls::stream<float> tb_in_Ts;
	hls::stream<float> tb_out_Igd;
	hls::stream<float> tb_out_Igq;
	hls::stream<float> tb_out_Ig0;
	hls::stream<float> tb_out_Ugd;
	hls::stream<float> tb_out_Ugq;
	hls::stream<float> tb_out_Ug0;
	hls::stream<float> tb_out_theta;
	hls::stream<float> tb_out_freq;

	/* Test inputs*/
	float in_Iga;
	float in_Igb;
	float in_Igc;
	float in_Uga;
	float in_Ugb;
	float in_Ugc;
	float in_Ts = 0.00005;		// fs = 20kHz
	float Ug_angle = 0;			// change to non-zero to see how pll converges
	float Ig_angle = 0;
	float freq_in = two_pi*50;	// fgrid = 50Hz
	float wt = freq_in*in_Ts;

	/* Test outputs */
	float out_Igd;
	float out_Igq;
	float out_Ugd;
	float out_Ugq;
	float out_theta;
	float out_freq;

	/* Create csv file to store simulation data*/
	FILE *inputs, *outputs;
	inputs = fopen ("inputs.csv", "w+");
	outputs = fopen ("outputs.csv", "w+");
	fprintf(inputs, "%s", "Uga,Ugb,Ugc,Iga,Igb,Igc\n");
	fprintf(outputs, "%s", "Ugd,Ugq,Igd,Igq,theta,theta_in,freq,freq_in\n");

	/* Run model */
	for (int i = 0; i < ITERATIONS; i++)
	{
		in_Uga = 311*cos(Ug_angle);
		in_Ugb = 311*cos(Ug_angle-two_pi/3);
		in_Ugc = 311*cos(Ug_angle+two_pi/3);
		in_Iga = 5*cos(Ig_angle);
		in_Igb = 5*cos(Ig_angle-two_pi/3);
		in_Igc = 5*cos(Ig_angle+two_pi/3);

		tb_in_Uga.write(in_Uga);
		tb_in_Ugb.write(in_Ugb);
		tb_in_Ugc.write(in_Ugc);
		tb_in_Iga.write(in_Iga);
		tb_in_Igb.write(in_Igb);
		tb_in_Igc.write(in_Igc);
		tb_in_Ts.write(in_Ts);

		vitis_grid_sync(tb_in_Iga,tb_in_Igb,tb_in_Igc,tb_in_Uga,tb_in_Ugb,tb_in_Ugc,tb_in_Ts,tb_out_Igd,tb_out_Igq,tb_out_Ig0,tb_out_Ugd,tb_out_Ugq,tb_out_Ug0,tb_out_theta,tb_out_freq);

		out_Igd = tb_out_Igd.read();
		out_Igq = tb_out_Igq.read();
		out_Ugd = tb_out_Ugd.read();
		out_Ugq = tb_out_Ugq.read();
		out_theta = tb_out_theta.read();
		out_freq = tb_out_freq.read();

		fprintf(inputs,"%f,%f,%f,%f,%f,%f %s",in_Uga,in_Ugb,in_Ugc,in_Iga,in_Igb,in_Igc,"\n");
		fprintf(outputs,"%f,%f,%f,%f,%f,%f,%f,%f %s",out_Ugd,out_Ugq,out_Igd,out_Igq,out_theta,Ug_angle,out_freq,freq_in,"\n");

		if(i>400)freq_in = two_pi*55;	// freq step at 0.02s
		wt = freq_in*in_Ts;
		Ug_angle = remainder((Ug_angle+wt),two_pi);
		Ig_angle = remainder((Ig_angle+wt),two_pi);
	}

	fclose(inputs);
	fclose(outputs);

	printf("------ Test Passed ------\n");

	/* End */
	return 0;
}
